// Copyright (c) 2021 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#if !defined(GAME_TYPE)
  #error You need to define the GAME_TYPE struct
#elif !defined(GAME_INIT)
  #error You need to define the GAME_INIT method
#elif !defined(GAME_UPDATE)
  #error You need to define the GAME_UPDATE method
#elif !defined(GAME_DISPOSE)
  #error You need to define the GAME_DISPOSE method
#elif !defined(GAME_FILE)
  #error You need to define the GAME_FILE string
#endif

#include "../libs.h"
#include "../engine/engine.h"
#include "../locale/language.h"
#include GAME_FILE

typedef GAME_TYPE game_t;

/**
 * Initialize the game context.
 * 
 * @return True if successful, otherwise false.
 */
bool gameInit(game_t *game);

/**
 * Tick the main game loop.
 * 
 * @param platformDelta The platform tick delta between the last render.
 * @return True if successful, false if safe exit requested..
 */
bool gameUpdate(game_t *game, float platformDelta);

/**
 * Cleanup the game instance.
 */
void gameDispose(game_t *game);